@@ -40,6 +40,8 @@
 #include <openssl/pem.h>
 #include <openssl/rand.h>
 
+#include <strings.h>
+#include <string.h>
 #include <syslog.h>
 
 #include "dma.h"
@@ -77,7 +79,7 @@ init_cert_file(SSL_CTX *ctx, const char *path)
 }
 
 int
-smtp_init_crypto(int fd, int feature)
+smtp_init_crypto(int fd, int feature, struct smtp_features* features)
 {
 	SSL_CTX *ctx = NULL;
 #if (OPENSSL_VERSION_NUMBER >= 0x00909000L)
@@ -93,7 +95,12 @@ smtp_init_crypto(int fd, int feature)
 	SSL_library_init();
 	SSL_load_error_strings();
 
-	meth = TLSv1_client_method();
+	// Allow any possible version
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+	meth = TLS_client_method();
+#else
+	meth = SSLv23_client_method();
+#endif
 
 	ctx = SSL_CTX_new(meth);
 	if (ctx == NULL) {
@@ -118,8 +125,7 @@ smtp_init_crypto(int fd, int feature)
 		/* TLS init phase, disable SSL_write */
 		config.features |= NOSSL;
 
-		send_remote_command(fd, "EHLO %s", hostname());
-		if (read_remote(fd, 0, NULL) == 2) {
+		if (perform_server_greeting(fd, features) == 0) {
 			send_remote_command(fd, "STARTTLS");
 			if (read_remote(fd, 0, NULL) != 2) {
 				if ((feature & TLS_OPP) == 0) {
@@ -131,6 +137,7 @@ smtp_init_crypto(int fd, int feature)
 				}
 			}
 		}
+
 		/* End of TLS init phase, enable SSL_write/read */
 		config.features &= ~NOSSL;
 	}
@@ -155,7 +162,7 @@ smtp_init_crypto(int fd, int feature)
 
 	/* Open SSL connection */
 	error = SSL_connect(config.ssl);
-	if (error < 0) {
+	if (error != 1) {
 		syslog(LOG_ERR, "remote delivery deferred: SSL handshake failed fatally: %s",
 		       ssl_errstr());
 		return (1);
